# -*- coding: utf-8 -*-
import os, logging, datetime, inspect, time
from config import default_config

logger_dict = {'info': logging.INFO,
               'warning': logging.WARNING,
               'critical': logging.CRITICAL,
               'debug': logging.DEBUG,
               'error': logging.ERROR}

project_name = 'factor-choice-server'


class MLCLogger(object):
    def __init__(self, logger_name, log_level='info',
                 log_format=None, log_dir=None):
        default_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        log_format = default_format if log_format is None else log_format
        formatter = logging.Formatter(log_format)
        self.logger = logging.getLogger(logger_name)
        console = logging.StreamHandler()
        console.setFormatter(formatter)
        self.logger.addHandler(console)
        self.set_level(log_level)
        if log_dir is not None:
            self._config_file(log_level=log_level, logger_name=logger_name,
                              log_dir=log_dir, log_format=log_format)

    def _config_file(self, log_level, logger_name, log_format,
                     log_dir):
        dir_name = os.path.join(log_dir, logger_name, log_level)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)
        filename = datetime.datetime.now().date().strftime('%Y-%m-%d') + '.log'
        logging.basicConfig(level=logger_dict[log_level.lower()],
                            format=log_format,
                            datefmt='%m-%d %H:%M',
                            filename=os.path.join(dir_name, filename),
                            filemode='a')

    def set_level(self, log_level):
        self.logger.setLevel(logger_dict[log_level.lower()])

    def info(self, msg):
        self.logger.info(msg)

    def warning(self, msg):
        self.logger.warning(msg)

    def critical(self, msg):
        self.logger.critical(msg)

    def debug(self, msg):
        self.logger.debug(msg)

    def error(self, msg):
        self.logger.error(msg)


class RDLogger(object):
    def __init__(self, logger_name, log_level='info'):
        self._logger_name = logger_name
        self.logger = logging.getLogger(logger_name)
        console = logging.StreamHandler()
        self.logger.addHandler(console)
        self.set_level(log_level)

    def set_level(self, log_level):
        self.logger.setLevel(logger_dict[log_level.lower()])

    def message(self, level, message):
        _, file_name, line_no, function_name, _, _ = inspect.stack()[2]
        return "[%s line:%s] %s - %s - %s - %s" % (file_name, line_no,
                                                   time.strftime('%Y-%m-%d %H:%M:%S', time.localtime()),
                                                   self._logger_name, level.upper(), message)

    def info(self, msg):
        self.logger.info(self.message('info', msg))

    def warning(self, msg):
        self.logger.info(self.message('warning', msg))

    def critical(self, msg):
        self.logger.info(self.message('critical', msg))

    def debug(self, msg):
        self.logger.info(self.message('debug', msg))

    def error(self, msg):
        self.logger.info(self.message('error', msg))


class LoggerFactory(object):
    @classmethod
    def create_engine(self, name='custom'):
        if name == 'rd':
            return RDLogger(project_name, log_dir=default_config.log_dir)
        else:
            return MLCLogger(project_name, log_dir=default_config.log_dir)


log = LoggerFactory.create_engine()

if __name__ == '__main__':
    a = {'a': '1111'}
    log.info((a['a']+'12'))
